﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Shapes;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.UI.Core;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.System;
using Windows.UI.Popups;
using System.Threading;
using SharpChip8;
using SharpChip8.Core;
using System.Threading.Tasks;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

namespace SharpChip8Metro
{
    /// <summary>
    /// An empty page that can be used on its own or navigated to within a Frame.
    /// </summary>
    public sealed partial class MainPage : Page
    {
        Chip8 _chip8;
        CoreDispatcher _dispatcher;

        public MainPage()
        {
            this.InitializeComponent();
            this._chip8 = App.Chip8;
            

            _dispatcher = Windows.UI.Core.CoreWindow.GetForCurrentThread().Dispatcher;
            Pixel.PixelWidth = (int)(canvasRenderer.ActualWidth / 64);
            Pixel.PixelHeight = (int)(canvasRenderer.ActualHeight / 32);
            this._chip8.Reset();
        }

        private async void InitializeEmulation()
        {
            while (_chip8.Cpu.Running && !_chip8.Cpu.WaitForInput)
            {
                await UpdateAndDraw();
            }

        }

        private bool FakeInit()
        {
            int width = (int)(canvasRenderer.ActualWidth / 64);
            int height = (int)(canvasRenderer.ActualHeight / 32);

            for (int x = 0; x < 64; x++)
            {
                for (int y = 0; y < 32; y++)
                {
                    int px = x * width;
                    int py = y * height;
                    Rectangle pixel = new Rectangle();
                    
                    SolidColorBrush brush = new SolidColorBrush();

                    if (x % (y + 1) == 0)
                        brush.Color = Colors.Black;
                    else
                        brush.Color = Colors.White;

                    pixel.Fill = brush;
                    pixel.Width = width;
                    pixel.Height = height;
                    
                    Canvas.SetLeft(pixel, px);
                    Canvas.SetTop(pixel, py);
                    
                    canvasRenderer.Children.Add(pixel);
                }
            }
            return true;
        }

        /// <summary>
        /// Invoked when this page is about to be displayed in a Frame.
        /// </summary>
        /// <param name="e">Event data that describes how this page was reached.  The Parameter
        /// property is typically used to configure the page.</param>
        protected override async void OnNavigatedTo(NavigationEventArgs e)
        {
            //FakeInit();
        }

        private void ButtonStartEmulation_Click(object sender, RoutedEventArgs e)
        {
            _chip8.Start();
            
        }

        private void ButtonPauseEmulation_Click(object sender, RoutedEventArgs e)
        {
            _chip8.Stop();
        }

        private void ButtonStopEmulation_Click(object sender, RoutedEventArgs e)
        {
            _chip8.Cpu.Running = false;
            _chip8.Cpu.Reset();
            canvasRenderer.Children.Clear();
        }

        private async void buttonLoadRom_Click(object sender, RoutedEventArgs e)
        {
            FileOpenPicker openPicker = new FileOpenPicker();
            openPicker.ViewMode = PickerViewMode.List;
            openPicker.SuggestedStartLocation = PickerLocationId.DocumentsLibrary;
            openPicker.FileTypeFilter.Add(".ch8");
            openPicker.FileTypeFilter.Add(".ck8");

            StorageFile file = await openPicker.PickSingleFileAsync();

            if (file != null)
            {
                _chip8.LoadRomFromFile(file);
            }
            _chip8.Start();
            InitializeEmulation();
        }

        private void DrawPixel(Pixel pixel)
        {
            SolidColorBrush brush = new SolidColorBrush();

           
            int px = pixel.X;
            int py = pixel.Y;
            Rectangle canvasPixel = new Rectangle();


            if (pixel.Color == PixelColor.Black)
                brush.Color = Colors.Black;
            else
                brush.Color = Colors.White;

            canvasPixel.Fill = brush;
            canvasPixel.Width = pixel.Width;
            canvasPixel.Height = pixel.Height;

            Canvas.SetLeft(canvasPixel, px);
            Canvas.SetTop(canvasPixel, py);

            canvasRenderer.Children.Add(canvasPixel);
        }

        private async Task<bool> UpdateAndDraw()
        {
            _chip8.Update();

            // Todo : ajouter un flag qui indiquera QUAND déssiner
            if (_chip8.Cpu.Running)
            {
                canvasRenderer.Children.Clear();
                for (int i = 0; i < SharpChip8.Core.Screen.InternalWidth; i++)
                    for (int j = 0; j < SharpChip8.Core.Screen.InternalHeight; j++)
                        DrawPixel(_chip8.Screen.Pixels[i][j]);
            }
            await Task.Delay(SharpChip8.Core.Cpu.WaitTime);

            return _chip8.Cpu.Running;
        }

        private void MainPage_KeyDown(object sender, KeyRoutedEventArgs e)
        {
            if (e.Key == VirtualKey.Escape && _chip8.Cpu.Running)
            {
                // Demander avant de quitter
            }

            byte opcode_b3 = (byte)((_chip8.Cpu.CurrentOpcode & 0x0F00) >> 8);

            switch (e.Key)
            {
                case VirtualKey.NumberPad0:
                    _chip8.Cpu.Input.Keys[0] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x0;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad1:
                    _chip8.Cpu.Input.Keys[1] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x1;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad2:
                    _chip8.Cpu.Input.Keys[2] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x2;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad3:
                    _chip8.Cpu.Input.Keys[3] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x3;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad4:
                    _chip8.Cpu.Input.Keys[4] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x4;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad5:
                    _chip8.Cpu.Input.Keys[5] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x5;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad6:
                    _chip8.Cpu.Input.Keys[6] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x6;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad7:
                    _chip8.Cpu.Input.Keys[7] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x7;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad8:
                    _chip8.Cpu.Input.Keys[8] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x8;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.NumberPad9:
                    _chip8.Cpu.Input.Keys[9] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0x9;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.A:
                    _chip8.Cpu.Input.Keys[10] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xA;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.Z:
                    _chip8.Cpu.Input.Keys[11] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xB;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.E:
                    _chip8.Cpu.Input.Keys[12] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xC;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.Q:
                    _chip8.Cpu.Input.Keys[13] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xD;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.S:
                    _chip8.Cpu.Input.Keys[14] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xE;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                case VirtualKey.D:
                    _chip8.Cpu.Input.Keys[15] = 0x1;
                    if (_chip8.Cpu.WaitForInput)
                    {
                        _chip8.Cpu.V[opcode_b3] = 0xF;
                        _chip8.Cpu.WaitForInput = false;
                    }
                    break;
                default:
                    break;
            }
        }

        private void MainPage_KeyUp(object sender, KeyRoutedEventArgs e)
        {
            switch (e.Key)
            {
                case VirtualKey.NumberPad0: _chip8.Cpu.Input.Keys[0] = 0x0; break;
                case VirtualKey.NumberPad1: _chip8.Cpu.Input.Keys[1] = 0x0; break;
                case VirtualKey.NumberPad2: _chip8.Cpu.Input.Keys[2] = 0x0; break;
                case VirtualKey.NumberPad3: _chip8.Cpu.Input.Keys[3] = 0x0; break;
                case VirtualKey.NumberPad4: _chip8.Cpu.Input.Keys[4] = 0x0; break;
                case VirtualKey.NumberPad5: _chip8.Cpu.Input.Keys[5] = 0x0; break;
                case VirtualKey.NumberPad6: _chip8.Cpu.Input.Keys[6] = 0x0; break;
                case VirtualKey.NumberPad7: _chip8.Cpu.Input.Keys[7] = 0x0; break;
                case VirtualKey.NumberPad8: _chip8.Cpu.Input.Keys[8] = 0x0; break;
                case VirtualKey.NumberPad9: _chip8.Cpu.Input.Keys[9] = 0x0; break;
                case VirtualKey.A: _chip8.Cpu.Input.Keys[10] = 0x0; break;
                case VirtualKey.Z: _chip8.Cpu.Input.Keys[11] = 0x0; break;
                case VirtualKey.E: _chip8.Cpu.Input.Keys[12] = 0x0; break;
                case VirtualKey.Q: _chip8.Cpu.Input.Keys[13] = 0x0; break;
                case VirtualKey.S: _chip8.Cpu.Input.Keys[14] = 0x0; break;
                case VirtualKey.D: _chip8.Cpu.Input.Keys[15] = 0x0; break;
                default:
                    break;
            }
        }
    }
}
